package org.util;

import java.io.File;
import java.util.Comparator;

public class FileComparator implements Comparator<File> {

	public int compare(File arg0, File arg1) {
        if (null == arg0 && null == arg1) 
        	return 0;
        if (null == arg0) 
        	return -1;
        if (null == arg1) 
        	return 1;
        if (arg0 instanceof File && arg1 instanceof File)
            return compare(arg0.getName(), arg1.getName());
        return 0;
	}
	
	public int compare(String s1, String s2) {
		if((s1 == null) && (s2 == null)) return 0;
		else if(s1 == null) return -1;
		else if(s2 == null) return 1;

		if((s1.length() <= 0) && (s2.length() <= 0)) return 0;
		else if(s1.length() <= 0) return -1;
		else if(s2.length() <= 0) return 1;

		//special case
		boolean sp1 = Character.isLetterOrDigit(s1.charAt(0));
		boolean sp2 = Character.isLetterOrDigit(s2.charAt(0));
		if(sp1 && !sp2) return 1;
		if(!sp1 && sp2) return -1;

		int i1 = 0, i2 = 0; //current index
		int r = 0; // temp result
		char c1, c2;
		boolean letter1, letter2;

		while(true)
		{
			c1 = s1.charAt(i1);
			c2 = s2.charAt(i2);
			sp1 = Character.isDigit(c1);
			sp2 = Character.isDigit(c2);
			if(!sp1 && !sp2)
			{
				letter1 = Character.isLetter(c1);
				letter2 = Character.isLetter(c2);

				if(letter1 && letter2)
				{
					r = new Character(Character.toUpperCase(c1)).compareTo(new Character(Character.toUpperCase(c2)));
					if(r != 0) return r;
				}
				else if(!letter1 && !letter2)
				{
					r = new Character(c1).compareTo(new Character(c2));
					if(r != 0) return r;
				}
				else if(!letter1 && letter2)
				{
					return -1;
				}
				else if(letter1 && !letter2)
				{
					return 1;
				}
			}
			else if(sp1 && sp2)
			{
				r = CompareNum(s1, new RefInt(i1), s2, new RefInt(i2));
				if(r != 0) return r;
			}
			else if(sp1)
			{
				return -1;
			}
			else if(sp2)
			{
				return 1;
			}
			i1++;
			i2++;
			if((i1 >= s1.length()) && (i2 >= s2.length()))
			{
				return 0;
			}
			else if(i1 >= s1.length())
			{
				return -1;
			}
			else if(i2 >= s2.length())
			{
				return 1;
			}
		}
	}

	private int CompareNum(String s1, RefInt i1, String s2, RefInt i2)
	{
		RefInt nzStart1 = new RefInt(i1.getValue());
		RefInt nzStart2 = new RefInt(i2.getValue()); // nz = non zero
		RefInt end1 = new RefInt(i1.getValue());
		RefInt end2 = new RefInt(i2.getValue());

		ScanNumEnd(s1, i1.getValue(), end1, nzStart1);
		ScanNumEnd(s2, i2.getValue(), end2, nzStart2);
		int start1 = i1.getValue(); i1.setValue(end1.getValue() - 1);
		int start2 = i2.getValue(); i2.setValue(end2.getValue() - 1);

		int nzLength1 = end1.getValue() - nzStart1.getValue();
		int nzLength2 = end2.getValue() - nzStart2.getValue();

		if(nzLength1 < nzLength2) return -1;
		else if(nzLength1 > nzLength2) return 1;

		for(int j1 = nzStart1.getValue(),j2 = nzStart2.getValue(); j1 <= i1.getValue(); j1++,j2++)
		{
			int r = new Character(s1.charAt(j1)).compareTo(new Character(s2.charAt(j2)));
			if(r != 0) return r;
		}
		// the nz parts are equal
		int length1 = end1.getValue() - start1;
		int length2 = end2.getValue() - start2;
		if(length1 == length2) return 0;
		if(length1 > length2) return -1;
		return 1;
	}

	//lookahead
	private void ScanNumEnd(String s, int start, RefInt end, RefInt nzStart)
	{
		nzStart.setValue(start);
		end.setValue(start);
		boolean countZeros = true;
		while(Character.isDigit(s.charAt(end.getValue())))
		{
			if(countZeros && s.charAt(end.getValue()) == ('0'))
			{
				nzStart.add(1);
			}
			else countZeros = false;
			end.add(1);
			if(end.getValue() >= s.length()) break;
		}
	}
	
	private class RefInt {
		private int value;
		public RefInt(int i) {
			value = i;
		}
		public void setValue(int i) {
			value = i;
		}
		
		public int getValue() {
			return value;
		}
		
		public void add(int sum) {
			value += sum;
		}
	}
}
